#include <fstream.h>
#include <vector>
#include <algorithm>
#include <math.h>
#include <stdlib.h>

using namespace std;

enum PL2SP_ALGO{NAIVE_ALGO, TCG_ALGO};

class Pl2SP
{
  private:
    vector<double> _xloc;
    vector<double> _yloc;
    vector<double> _widths;
    vector<double> _heights;

    vector<unsigned> _X;
    vector<unsigned> _Y;

    int _cnt;
  public:
    Pl2SP(vector<double>& xloc, vector<double>& yloc, vector<double>& widths,
          vector<double>& heights, PL2SP_ALGO whichAlgo);

    ~Pl2SP() {}
    
    void naiveAlgo(void);
    void TCGAlgo(void);

    //Floyd Marshal to find TCG
    void TCG_FM(vector< vector <bool> >& TCGMatrixHoriz, 
		vector< vector <bool> >& TCGMatrixVert);

    //DP to find TCG
    void TCG_DP(vector< vector <bool> >& TCGMatrixHoriz, 
		vector< vector <bool> >& TCGMatrixVert);
    void TCGDfs(vector< vector <bool> >& TCGMatrix, 
		vector< vector <bool> >& adjMatrix, int v, 
		vector<int>& pre);

    vector<unsigned>& getXSP(void)
     { return _X; }
     
    vector<unsigned>& getYSP(void)
     { return _Y; }

    void print(void);

};

struct RowElem
{
  unsigned index;
  double xloc;
};


struct less_mag 
{
  bool operator()(const RowElem &elem1, const RowElem &elem2) 
  { return(elem1.xloc < elem2.xloc); }
};

class SPXRelation
{
  vector< vector<bool> >& TCGMatrixHoriz;
  vector< vector<bool> >& TCGMatrixVert;

  public:
  SPXRelation(vector< vector<bool> >& TCGMatrixHorizIP, 
	      vector< vector<bool> >& TCGMatrixVertIP) : 
    TCGMatrixHoriz(TCGMatrixHorizIP), TCGMatrixVert(TCGMatrixVertIP)
    {}

  bool operator()(unsigned i, unsigned j) 
    {
      if(TCGMatrixHoriz[i][j] == 1)
	return 1;
      else if(TCGMatrixHoriz[j][i] == 1)
	return 0;
      else if(TCGMatrixVert[j][i] == 1)
	return 1;
      else if(TCGMatrixVert[i][j] == 1)
	return 0;
      else
	{
	  //cout<<"ERROR IN PL2SP SPX "<<i<<"\t"<<j<<endl;
	  if(i<j)
	    return 1;
	  else
	    return 0;
	}
    }
};

class SPYRelation
{
  vector< vector<bool> >& TCGMatrixHoriz;
  vector< vector<bool> >& TCGMatrixVert;

  public:
  SPYRelation(vector< vector<bool> >& TCGMatrixHorizIP, 
	      vector< vector<bool> >& TCGMatrixVertIP) : 
    TCGMatrixHoriz(TCGMatrixHorizIP), TCGMatrixVert(TCGMatrixVertIP)
    {}
  bool operator()(unsigned i, unsigned j) 
    {
      if(TCGMatrixHoriz[i][j] == 1)
	return 1;
      else if(TCGMatrixHoriz[j][i] == 1)
	return 0;
      else if(TCGMatrixVert[j][i] == 1)
	return 0;
      else if(TCGMatrixVert[i][j] == 1)
	return 1;
      else
	{
	  //cout<<"ERROR IN PL2SP SPY "<<i<<"\t"<<j<<endl;
	  if(i<j)
	    return 1 ;
	  else
	    return 0;
	}
    }
};
