#ifndef COMMAND_LINE_H
#define COMMAND_LINE_H
#include <string>

class Command_Line
{
	public:
		Command_Line (int argc, const char *argv[]);
				
		// public member variables to store params
		bool getSeed, budgetTime;
		bool softBlocks;
		bool initQP;     //initialize a QP soln
		char inFileName[200];
		char outPlFile [200];
		char capoPlFile [200];
		char capoBaseFile [200];
		char baseFile [200];

		int seed;      //fixed seed
		int iterations;//number of runs
		double seconds;
                bool plot;             //plot to out.gpl
		bool plotNoNets;       //do not plot nets 
		bool plotNoSlacks;     //do not plot slacks
		bool plotNoNames;      //do not plot names
                bool savePl;           //save .pl file as output
		bool saveCapoPl;       //save .pl file in Capo format
		bool saveCapo;         //save files in Capo format
		bool save;             //save files in bookshelf format

		bool takePl;           //takes a Placement and converts it 
				       //to sequence pair for initial soln
		bool solveMulti;       //use multilevel heirarchy
		bool clusterPhysical;  //use physical hierarchy instead of 
		                       //closest node
		bool solveTop;         //solve only top level of heirarchy
		bool compact;          //compact final soln
		double maxWSHier;      //maximum WS for heirarchical blocks
		                       //default 15%
		//Annealer performance parameters next
		double timeInit;     //initial temperature default 30000
		double timeCool;     //cooling temperature default 0.01
		double startTime;    //default to timeInit
		           //(this is the time the area & WL are normalized to)
		double reqdAR;       //required Aspect Ratio of fixed outline
				     //default -9999(means no fixed outline
				     //desired)
		double maxWS;        //if fixed-outline then maximum whitespace
		                     //acceptable
		bool minWL;          //whether HPWL minimization desired
		bool initCompact;    //whether to use compaction to generate
		                     //initial SP
		
	public:
		// Print usage info. to cerr
		void printHelp ();
		//print the Annealer params
		void printAnnealerParams();
		
};

#endif // COMMAND_LINE_H
